﻿Imports System.Data
Imports System.IO
Imports Telerik.Documents.SpreadsheetStreaming
Imports Telerik.Web.UI
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf
Imports Telerik.Windows.Documents.Spreadsheet.Model

Partial Class DefaultVB
    Inherits System.Web.UI.Page

    Private Property SessionKey As String
    Private Property colNum As Integer
    Private Property rowNum As Integer
    Private Property FileExtension As String
    Private Property FileName As String
    Private Property ResponseContentType As String

    Private Property SessionDataSource As DataTable
        Get

            If Not IsPostBack OrElse Session(SessionKey) Is Nothing Then
                Session(SessionKey) = OrdersTable()
            End If

            Return CType(Session(SessionKey), DataTable)
        End Get
        Set(ByVal value As DataTable)
            Session(SessionKey) = value
        End Set
    End Property

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
        SessionKey = "MySessionkey"
        colNum = 10
        rowNum = 100000
    End Sub

    Protected Sub RadGrid1_NeedDataSource(ByVal sender As Object, ByVal e As GridNeedDataSourceEventArgs)
        RadGrid1.DataSource = SessionDataSource
    End Sub

    Protected Sub btnXLSX_Click(ByVal sender As Object, ByVal e As EventArgs)
        SpreadStreamProcessingForXLSXAndCSV(SessionDataSource)
    End Sub

    Protected Sub btnCSV_Click(ByVal sender As Object, ByVal e As EventArgs)
        SpreadStreamProcessingForXLSXAndCSV(SessionDataSource, SpreadDocumentFormat.Csv)
    End Sub

    Protected Sub btnPDF_Click(ByVal sender As Object, ByVal e As EventArgs)
        SpreadProcessingForPDF(SessionDataSource)
    End Sub

    Private Function OrdersTable() As DataTable
        Dim dt As DataTable = New DataTable()

        For i As Integer = 0 To colNum - 1
            dt.Columns.Add(New DataColumn("C" & i, GetType(String)))
        Next

        For rowIndex As Integer = 0 To rowNum - 1
            Dim row As DataRow = dt.NewRow()

            For colIndex As Integer = 0 To dt.Columns.Count - 1
                row(colIndex) = String.Format("C{0}R{1}", colIndex, rowIndex)
            Next

            dt.Rows.Add(row)
        Next

        Return dt
    End Function

    Private Sub SpreadProcessingForPDF(ByVal dt As DataTable)
        Dim colCount As Integer = dt.Columns.Count
        Dim rowCount As Integer = dt.Rows.Count
        Dim spWorkbook As Workbook = New Workbook()
        spWorkbook.History.IsEnabled = False
        Dim spWorksheet As Worksheet = spWorkbook.Worksheets.Add()

        For rowIndex As Integer = 0 To rowCount - 1
            Dim dr As DataRow = dt.Rows(rowIndex)

            For colIndex As Integer = 0 To colCount - 1

                If rowIndex = 0 Then
                    Dim colName As String = dt.Columns(colIndex).ColumnName
                    spWorksheet.Cells(rowIndex, colIndex).SetValue(colName)
                Else
                    Dim dbValue = dr(colIndex).ToString()
                    spWorksheet.Cells(rowIndex, colIndex).SetValue(dbValue)
                    Dim cellValue As String = spWorksheet.Cells(1, 1).GetValue().Value.RawValue
                End If
            Next
        Next

        Dim pdfProvider As PdfFormatProvider = New PdfFormatProvider()
        Dim output As Byte() = pdfProvider.Export(spWorkbook)
        FileExtension = "pdf"
        FileName = RadGrid1.ExportSettings.FileName
        ResponseContentType = "application/pdf"
        WriteFileToResponse(output)
    End Sub

    Private Sub SpreadStreamProcessingForXLSXAndCSV(ByVal dt As DataTable, ByVal Optional docFormat As SpreadDocumentFormat = SpreadDocumentFormat.Xlsx, ByVal Optional sheetName As String = "Sheet1")
        Using stream As MemoryStream = New MemoryStream()

            Using workbook As IWorkbookExporter = SpreadExporter.CreateWorkbookExporter(docFormat, stream)

                Using worksheetExporter As IWorksheetExporter = workbook.CreateWorksheetExporter(sheetName)

                    For i As Integer = 0 To dt.Columns.Count - 1

                        Using columnExporter As IColumnExporter = worksheetExporter.CreateColumnExporter()
                            columnExporter.SetWidthInPixels(100)
                        End Using
                    Next

                    ExportHeaderRows(worksheetExporter, dt)

                    For Each row As DataRow In dt.Rows

                        Using rowExporter As IRowExporter = worksheetExporter.CreateRowExporter()

                            For Each item In row.ItemArray
                                Dim normalFormat As SpreadCellFormat = New SpreadCellFormat()
                                normalFormat.FontSize = 10
                                normalFormat.VerticalAlignment = SpreadVerticalAlignment.Center
                                normalFormat.HorizontalAlignment = SpreadHorizontalAlignment.Center

                                Using cellExporter As ICellExporter = rowExporter.CreateCellExporter()
                                    cellExporter.SetValue(item.ToString())
                                    cellExporter.SetFormat(normalFormat)
                                End Using
                            Next
                        End Using
                    Next
                End Using
            End Using

            Dim output As Byte() = stream.ToArray()

            If docFormat = SpreadDocumentFormat.Csv Then
                FileExtension = "csv"
                FileName = RadGrid1.ExportSettings.FileName
                ResponseContentType = "text/csv"
            ElseIf docFormat = SpreadDocumentFormat.Xlsx Then
                FileExtension = "xlsx"
                FileName = RadGrid1.ExportSettings.FileName
                ResponseContentType = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
            End If

            WriteFileToResponse(output)
        End Using
    End Sub

    Private Sub ExportHeaderRows(ByVal worksheetExporter As IWorksheetExporter, ByVal dt As DataTable)
        Using rowExporter As IRowExporter = worksheetExporter.CreateRowExporter()
            Dim HeaderRowHeight As Double = 50
            rowExporter.SetHeightInPoints(HeaderRowHeight)
            Dim format As SpreadCellFormat = New SpreadCellFormat()
            format.IsBold = True
            format.Fill = SpreadPatternFill.CreateSolidFill(New SpreadColor(142, 196, 65))
            format.ForeColor = New SpreadThemableColor(New SpreadColor(255, 255, 255))
            format.HorizontalAlignment = SpreadHorizontalAlignment.Center
            format.VerticalAlignment = SpreadVerticalAlignment.Center

            For i As Integer = 0 To dt.Columns.Count - 1

                Using cellExporter As ICellExporter = rowExporter.CreateCellExporter()
                    cellExporter.SetFormat(format)
                    cellExporter.SetValue(dt.Columns(i).ColumnName)
                End Using
            Next
        End Using
    End Sub

    Private Sub WriteFileToResponse(ByVal content As Byte())
        Response.ContentType = ResponseContentType
        Response.Headers.Remove("Content-Disposition")
        Response.AppendHeader("Content-Disposition", String.Format("attachment; filename={0}.{1}", FileName, FileExtension))
        Response.BinaryWrite(content)
        Response.[End]()
    End Sub
End Class